
{-

  The same problem as we had with filter. I would like an element x of type A
  such that f x reduces to e (and that the typechecker can see this). We should
  be able to construct this element from a y and a proof that f y == e.

    f : A -> B   e : B
    ------------------
    computesTo f e : A

    f (computesTo f e) --> e

  Termination issues?

          f x == f x
    -----------------------
    x := computesTo f (f x)   * recursive (bad!)

  We can restrict the right hand side to be a constructor application.

  Can we emulate this? Lift functions to operate on sums?
-}

data Argument (A,B:Set) : Set where
  argument : A -> Argument A B
  result   : B -> Argument A B

lift (A,B:Set) : (A -> B) -> Argument A B -> B
lift f (argument x) = f x
lift f (result y)   = y

data Compute {A,B:Set}(f:A -> B) : Argument A B -> Set where
  computesTo (b:B) : Compute f (result b)

{-
    No. "argument x" is never going to be equal to "result y" however much f x == y.

    It has to be hard-wired in other words.

    Maybe it would be simpler to just add rewriting. Whenever you pattern match on e
    with pattern p you get a rewrite rule e --> p.
    
    If you have eq : f x == false you can do

      foo x with f x =>
	  x true  = discriminate eq
	  x false = ...
-}

