-- Properties (subjects of Agda -> FOL translation)
-- Authors:: Andreas Abel, Ulf Norell
-- Created:: 2004-11-16

-- Prop provides codes for quantifier free propositions.

sort Prop : Type where
  Pi	Set Prop -> Prop
  Sigma Set Prop -> Prop

-- atoms

Eq : (A : Set) -> A -> A -> Prop

-- connectives

data True : Prop where
  tt : True

data False : Prop where

(==>) : Prop -> Prop -> Prop
p ==> q = p -> q

(<==) : Prop -> Prop -> Prop
p <== q = q ==> p

Not : Prop -> Prop
Not P = P ==> False

(/\) : Prop -> Prop -> Prop
P /\ Q = { P ; Q }

-- Classical or
(\/) : Prop -> Prop -> Prop
P \/ Q = Not (Not P /\ Not Q)

(<=>) : Prop -> Prop -> Prop
P <=> Q = (P ==> Q) /\ (Q ==> P)

ForAll : (A:Set) -> (A -> Prop) -> Prop
ForAll A B = (x:A) -> B x

Exists : (A:Set) -> (A -> Prop) -> Prop
Exists A B = Not (ForAll A (\x -> Not (B x)))

infixl 4 /\
infixl 3 \/
infixr 2 ==>, <==
infix 1 <=>

